@@ -110,9 +110,63 @@ class dataclass_t:
     three: int = dtcl.field(init=False, default=3)
 
 
-def CheckJsonization(instance: object, /, *, builders=None) -> bool:
+class class_t:
+    one: int
+    two: int
+    three: int
+
+    def __init__(self, one: int, two: int, three: int, /) -> None:
+        self.one = one
+        self.two = two
+        self.three = three
+
+    def __eq__(self, other, /) -> bool:
+        if type(other) is type(self):
+            return ClassAsTuple(other) == ClassAsTuple(self)
+
+        return False
+
+
+def NewClassFromTuple(all_three: tuple[int, int, int], /) -> class_t:
+    return class_t(*all_three)
+
+
+def ClassAsTuple(instance: class_t, /) -> tuple[int, int, int]:
+    return instance.one, instance.two, instance.three
+
+
+class class_w_special_methods_t:
+    one: int
+    two: int
+    three: int
+
+    def __init__(self, one: int, two: int, three: int, /) -> None:
+        self.one = one
+        self.two = two
+        self.three = three
+
+    @classmethod
+    def __NewFromJsonDescription__(
+        cls, all_three: tuple[int, int, int], /
+    ) -> class_w_special_methods_t:
+        return cls(*all_three)
+
+    def __DescriptionForJSON__(self) -> tuple[int, int, int]:
+        return self.one, self.two, self.three
+
+    def __eq__(self, other, /) -> bool:
+        if type(other) is type(self):
+            return other.__DescriptionForJSON__() == self.__DescriptionForJSON__()
+
+        return False
+
+
+def CheckJsonization(instance: object, /, *, descriptors=None, builders=None) -> bool:
     """"""
-    jsoned = jsny.JsonStringOf(instance)
+    if hasattr(instance, "AsJsonString"):
+        jsoned = instance.AsJsonString()
+    else:
+        jsoned = jsny.JsonStringOf(instance, descriptors=descriptors)
 
     return DecodeAndCompare(jsoned, instance, builders=builders)
 
@@ -128,10 +182,14 @@ def DecodeAndCompare(jsoned: str, instance: object, /, *, builders=None) -> bool
     if type(instance) is type(rebuilt):
         if isinstance(instance, io_bytes_t):
             equality = instance.getvalue() == rebuilt.getvalue()
-        elif isinstance(instance, array_t):
+        elif (nmpy is not None) and isinstance(instance, array_t):
             equality = nmpy.array_equal(instance, rebuilt) and (
                 instance.dtype == rebuilt.dtype
             )
+        elif (nmpy is not None) and nmpy.issubclass_(
+            type(instance), (nmpy.bool_, nmpy.number, nmpy.character)
+        ):
+            equality = (instance == rebuilt) and (instance.dtype == rebuilt.dtype)
         elif (_SPARSE_ARRAY_CLASSES is not None) and isinstance(
             instance, _SPARSE_ARRAY_CLASSES
         ):
@@ -140,7 +198,7 @@ def DecodeAndCompare(jsoned: str, instance: object, /, *, builders=None) -> bool
             equality = nmpy.array_equal(instance_as_array, rebuilt_as_array) and (
                 instance_as_array.dtype == rebuilt_as_array.dtype
             )
-        elif isinstance(instance, graph_t):
+        elif (grph is not None) and isinstance(instance, graph_t):
             equality = grph.utils.misc.graphs_equal(instance, rebuilt)
         else:
             equality = instance == rebuilt
@@ -182,6 +240,9 @@ if __name__ == "__main__":
     instance_ = named_tuple_t(string="string", integer=123)
     CheckJsonization(instance_)
 
+    instance_ = complex(1, 2)
+    CheckJsonization(instance_)
+
     instance_ = date_t(2000, 12, 31)
     CheckJsonization(instance_)
     #
@@ -218,8 +279,67 @@ if __name__ == "__main__":
     instance_ = dataclass_t(10, 20)
     CheckJsonization(instance_)
 
+    instance_ = class_t(1, 2, 3)
+    CheckJsonization(
+        instance_,
+        descriptors={"": ClassAsTuple},
+        builders={"": NewClassFromTuple},
+    )
+
+    instance_ = class_w_special_methods_t(1, 2, 3)
+    CheckJsonization(instance_)
+
+    # Recursion test (appears as type "dict")
+    instance_ = {
+        enum_e.ONE: dataclass_t(10, 20),
+        time_t(23, 59, 1): [slice(0, 10, 2), path_t("/folder/subfolder/file.txt")],
+    }
+    CheckJsonization(instance_)
+
     if nmpy is not None:
-        instance_ = nmpy.array([[1, 2, 3], [4, 5, 6]], dtype=nmpy.uint8)
+        instance_ = nmpy.array([[1, 2, 3], [4, 5, 6]], dtype=bool)
+        CheckJsonization(instance_)
+        instance_ = nmpy.array([[1, 2, 3], [4, 5, 6]], dtype=nmpy.bool_)
+        CheckJsonization(instance_)
+
+        instance_ = nmpy.array([[1, 2, 3], [4, 5, 6]], dtype=int)
+        CheckJsonization(instance_)
+        instance_ = nmpy.array([[1, 2, 3], [4, 5, 6]], dtype=nmpy.uint16)
+        CheckJsonization(instance_)
+
+        instance_ = nmpy.array([[1, 2, 3], [4, 5, 6]], dtype=float)
+        CheckJsonization(instance_)
+        instance_ = nmpy.array([[1, 2, 3], [4, 5, 6]], dtype=nmpy.float64)
+        CheckJsonization(instance_)
+
+        instance_ = nmpy.array([[1, 2, 3], [4, 5, 6]], dtype=complex)
+        CheckJsonization(instance_)
+        instance_ = nmpy.array([[1, 2, 3], [4, 5, 6]], dtype=nmpy.complex_)
+        CheckJsonization(instance_)
+        instance_ = nmpy.array([[1, 2, 3], [4, 5, 6]], dtype=nmpy.complex128)
+        CheckJsonization(instance_)
+
+        instance_ = nmpy.array([["a", "b", "c"], ["d", "e", "f"]], dtype=str)
+        CheckJsonization(instance_)
+        instance_ = nmpy.array([["a", "b", "c"], ["d", "e", "f"]], dtype=nmpy.str_)
+        CheckJsonization(instance_)
+
+        instance_ = nmpy.bool_(True)
+        CheckJsonization(instance_)
+
+        instance_ = nmpy.uint16(1)
+        CheckJsonization(instance_)
+
+        instance_ = nmpy.float64(1)
+        CheckJsonization(instance_)
+
+        instance_ = nmpy.complex_(1.0 + 2.0j)
+        CheckJsonization(instance_)
+
+        instance_ = nmpy.complex128(1.0 + 2.0j)
+        CheckJsonization(instance_)
+
+        instance_ = nmpy.str_("a")
         CheckJsonization(instance_)
 
     if sprs is not None:
